# -*- coding: utf-8 -*-

from adsputils import ADSCelery, ADSTask
import unittest


class TestUpdateRecords(unittest.TestCase):

    def test_config(self):
        app = ADSCelery('test',local_config={
            'FOO': ['bar', {}]
            })
        self.assertEqual(app._config['FOO'], ['bar', {}])
        self.assertEqual(app.conf['FOO'], ['bar', {}])
        
        self.assertEqual(app.conf['CELERY_RESULT_SERIALIZER'], 'adsmsg')
        self.assertFalse(app._config.get('CELERY_RESULT_SERIALIZER', None))
    
    def test_app_task(self):
        
        class NewCelery(ADSCelery):
            def attempt_recovery(self, task, args=None, kwargs=None, einfo=None, retval=None):
                if 'Failing!' in str(retval):
                    # half the number of processed objects
                    first_half, second_half = args[0][0:len(args[0])/2], args[0][len(args[0])/2:]
                    # resubmit
                    args = list(args)
                    args[0] = first_half
                    task.apply(args=args, kwargs=kwargs)
                    args[0] = second_half
                    task.apply(args=args, kwargs=kwargs)
                    
        app = NewCelery('test',local_config={
            'FOO': ['bar', {}]
            })
        
        
        processed = []
        @app.task
        def test(batch):
            if len(batch) == 10:
                processed.append('Failure')
                raise Exception('Failing!')
            processed.append(batch)
        
        self.assertRaises(Exception, lambda: test.apply(args=(range(10),)))
        self.assertEqual(processed, ['Failure', 'Failure', [0,1,2,3,4], [5,6,7,8,9]])
        
        processed = []
        test.max_retries = 3
        self.assertRaises(Exception, lambda: test.apply(args=(range(10),)))
        self.assertEqual(processed, ['Failure', 'Failure', 'Failure', 'Failure', [0,1,2,3,4], [5,6,7,8,9]])
        
if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import sys
import os

import unittest
import json
import inspect
from mock import patch
from kombu import serialization
from adsputils import serializer
from adsmsg import BibRecord
from adsmsg.msg import Msg

class TestAdsOrcidCelery(unittest.TestCase):
    """
    Tests the JSON serializer of adsmsg protobufs
    """
    
    def setUp(self):
        unittest.TestCase.setUp(self)
        serialization.register('adsmsg', *serializer.register_args)
    
    
    def tearDown(self):
        unittest.TestCase.tearDown(self)
        serialization.unregister('adsmsg')
        
    
    def test_serializer(self):
        b = BibRecord(bibcode='bibcode')
        cls, data = b.dump()
        
        self.assertEqual('adsmsg.bibrecord.BibRecord', cls)
        self.assertEqual(data, '\n\x07bibcode') # let's hope the upstream adsmsg doesn't change impl...
        
        
        # dump using default 'json' serializer
        content_type, encoding, data = serialization.dumps(b)
        self.assertEqual('{"__adsmsg__": ["adsmsg.bibrecord.BibRecord", "CgdiaWJjb2Rl"]}',
                         data)
        
        # load using our deserializer
        o = serialization.loads(data, 'application/x-adsmsg', encoding)
        self.assertTrue(isinstance(o, BibRecord))
        self.assertEqual(o.bibcode, 'bibcode')

        
        # and arbitrarily nested objects should pose no difficulties
        test = [{'foo': b}, b, [[b]]]
        content_type, encoding, data = serialization.dumps(test)
        o = serialization.loads(data, 'application/x-adsmsg', encoding)
        
        for x in (o[0]['foo'], o[1], o[2][0][0]):
            self.assertTrue(isinstance(x, BibRecord))
            self.assertEqual(x.bibcode, "bibcode")


    def test_utf8(self):
        b = BibRecord(bibcode=u'\u01b5')
        ctype, enc, data = serialization.dumps(b)
        o = serialization.loads(data, 'application/x-adsmsg', 'utf-8')
        self.assertTrue(isinstance(o, BibRecord))
        self.assertEqual(o.bibcode, u'\u01b5')
        

if __name__ == '__main__':
    unittest.main()
